require "spec_helper"

describe Actionable::Recurrence do

  context "every day at 2:00 AM" do
    let(:every_day_at_2){ "0 2 * * *" }

    it "should parse good" do
      recurrence = Rufus::CronLine.new(every_day_at_2)
    end
  end

  context "every five minutes" do

    let(:every_five){ "*/5 * * * *" }
    let(:new_args){
      { recurrence_string:every_five, job_class: RecurringTestJob }
    }

    let(:recurrence) {
      Actionable::Recurrence.create(new_args)
    }

    it "should be instantiated with a recurrence string and a job class" do
      r = Actionable::Recurrence.new(new_args)
      expect(r).to be_valid
    end

    it "should refuse to save with a bad recurrence string" do
      new_args[:recurrence_string] = '* D * L * . *'
      r = Actionable::Recurrence.new(new_args)
      expect(r).to_not be_valid
    end

    it "should refuse to save with a non-existent class" do
      new_args[:job_class] = "NonExistentJob"
      r = Actionable::Recurrence.new(new_args)
      expect(r).to_not be_valid
    end


    it "should not add to the database until saved" do
      r = Actionable::Recurrence.new(new_args)
      expect(Actionable::Recurrence.count).to eq(0)
    end

    it "should add to the database afer being committed" do
      r = Actionable::Recurrence.new(new_args)
      r.save
      expect(Actionable::Recurrence.count).to eq(1)
    end

    it "should know its job class" do
      expect(recurrence.send(:job_class)).to eq(RecurringTestJob)
    end

    it "should create an actionable upon creation" do
      expect(Actionable::Action.count).to eq(0)
      Actionable::Recurrence.create(new_args)
      expect(Actionable::Action.count).to eq(1)
    end

    context "when time passes" do

      before do
        Resque.inline = true
        recurrence.valid?
        Timecop.travel(5.minutes.from_now)
      end

      it "should run the first action and create a new one on sweep" do
        expect(Actionable::Action.count).to eq(1)
        2.times { Actionable::Sweep.perform }
        expect(Actionable::Action.count).to eq(2)
      end

    end

    context "when no time passes" do

      before do
        Resque.inline = true
        recurrence.valid?
      end

      it "should not run the first action or create a new one" do
        expect(Actionable::Action.count).to eq(1)
        Actionable::Sweep.perform
        expect(Actionable::Action.count).to eq(1)
      end

    end

    context "over the course of an hour" do

      it "should create and perform the job 11 times" do
        RecurringTestJob.should_receive(:go).at_least(11).times
        Timecop.travel(1.hour.from_now.change(minutes:0))
        recurrence.valid?
        60.times do
          Actionable::Sweep.perform
          Timecop.travel(1.minute.from_now)
        end
        expect(Actionable::Action.count).to be_within(1).of(12)
      end

    end

  end

end
